/*
  Name: Sound.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This class provides a high level API for
	  loading and playing sounds of different formats
*/

#include <allegro.h>
#include <string>
#include <fstream>
using namespace std;

#include "Sound.h"

enum activeStates
{
	INACTIVE = 0,
	SAMPLEACTIVE = 1,
	MIDIACTIVE = 2,
};

Sound::Sound()
{
	sample = NULL;
	midi = NULL;
	active = INACTIVE;
}

Sound::Sound(string filename)
{
	Sound();
	load(filename);
}

Sound::~Sound()
{
	destroy();
}

bool Sound::load(string filename)
{
	destroy();
	//Load wav file
	if(
	(
		filename[filename.length() - 3] == 'w' &&
		filename[filename.length() - 2] == 'a' &&
		filename[filename.length() - 1] == 'v'
	)
	||
	(
		filename[filename.length() - 3] == 'W' &&
		filename[filename.length() - 2] == 'A' &&
		filename[filename.length() - 1] == 'V'
	))
	{
		sample = load_sample(filename.c_str());
		if(!sample)
			return false;

		active = SAMPLEACTIVE;
	}
	//Load midi
	else if(
		(
			filename[filename.length() - 3] == 'm' &&
			filename[filename.length() - 2] == 'i' &&
			filename[filename.length() - 1] == 'd'
		)
		||
		(
			filename[filename.length() - 4] == 'm' &&
			filename[filename.length() - 3] == 'i' &&
			filename[filename.length() - 2] == 'd' &&
			filename[filename.length() - 1] == 'i'
		))
	{
		midi = load_midi(filename.c_str());
		if(!midi)
			return false;

		active = MIDIACTIVE;
	}
	else
		return false;
	return true;
}

void Sound::play(int loop, int vol, int pan, int freq)
{
	if(active == MIDIACTIVE)
	{
		play_midi(midi, loop);
	}
	else if(active == SAMPLEACTIVE)
	{
		play_sample(sample, vol, pan, freq, loop);
	}

	this->loop = loop;
	this->vol = vol;
	this->pan = pan;
	this->freq = freq;
}

void Sound::stop()
{
	if(active == SAMPLEACTIVE)
	{
		stop_sample(sample);
	}
	else if(active == MIDIACTIVE)
	{
		stop_midi();
	}
}

void Sound::destroy()
{
	if(sample)
	{
		destroy_sample(sample);
		sample = NULL;
	}
	if(midi)
	{
		destroy_midi(midi);
		midi = NULL;
	}
	active = INACTIVE;
}

int Sound::isActive()
{
	return active;
}
